@@ -19,6 +19,12 @@
 #include <cairomm/context.h>
 #include <cairomm/private.h>
 
+/* M_PI is defined in math.h in the case of Microsoft Visual C++ */
+#if defined(_MSC_VER)
+#define _USE_MATH_DEFINES
+#include <math.h>
+#endif 
+
 namespace Cairo
 {
 
@@ -223,12 +229,18 @@ void Context::device_to_user_distance(double& dx, double& dy)
   check_object_status_and_throw_exception(*this);
 }
 
-void Context::clear_path()
+void Context::begin_new_path()
 {
   cairo_new_path(m_cobject);
   check_object_status_and_throw_exception(*this);
 }
 
+void Context::begin_new_sub_path()
+{
+  cairo_new_sub_path(m_cobject);
+  check_object_status_and_throw_exception(*this);
+}
+
 void Context::move_to(double x, double y)
 {
   cairo_move_to(m_cobject, x, y);
@@ -301,15 +313,15 @@ void Context::paint_with_alpha(double alpha)
   check_object_status_and_throw_exception(*this);
 }
 
-void Context::mask(const RefPtr<Pattern>& pattern)
+void Context::mask(const RefPtr<const Pattern>& pattern)
 {
-  cairo_mask(m_cobject, pattern->cobj());
+  cairo_mask(m_cobject, const_cast<cairo_pattern_t*>(pattern->cobj()));
   check_object_status_and_throw_exception(*this);
 }
 
-void Context::mask(const RefPtr<Surface>& surface, double surface_x, double surface_y)
+void Context::mask(const RefPtr<const Surface>& surface, double surface_x, double surface_y)
 {
-  cairo_mask_surface(m_cobject, surface->cobj(), surface_x, surface_y);
+  cairo_mask_surface(m_cobject, const_cast<cairo_surface_t*>(surface->cobj()), surface_x, surface_y);
   check_object_status_and_throw_exception(*this);
 }
 
@@ -597,13 +609,61 @@ Path* Context::copy_path_flat() const
   return new Path(cresult, true /* take ownership */); //The caller must delete it.
 }
 
-//TODO: Should this be const? Does cairo_append_path() take ownership?
 void Context::append_path(const Path& path)
 {
   cairo_append_path(m_cobject, const_cast<cairo_path_t*>(path.cobj()));
   check_object_status_and_throw_exception(*this);
 }
 
+void Context::push_group()
+{
+  cairo_push_group(m_cobject);
+  check_object_status_and_throw_exception(*this);
+}
+
+void Context::push_group_with_content(Content content)
+{
+  cairo_push_group_with_content(m_cobject, static_cast<cairo_content_t>(content));
+  check_object_status_and_throw_exception(*this);
+}
+
+RefPtr<Pattern> Context::pop_group()
+{
+  cairo_pattern_t* pattern = cairo_pop_group(m_cobject);
+  check_object_status_and_throw_exception(*this);
+  return RefPtr<Pattern>(new Pattern(pattern));
+}
+
+void Context::pop_group_to_source()
+{
+  cairo_pop_group_to_source(m_cobject);
+  check_object_status_and_throw_exception(*this);
+}
+
+RefPtr<Surface> Context::get_group_target()
+{
+  cairo_surface_t* surface = cairo_get_group_target(m_cobject);
+  // surface can be NULL if you're not between push/pop group calls
+  if (surface == NULL)
+  {
+    // FIXME: is this really the right way to handle this?
+    throw_exception(CAIRO_STATUS_NULL_POINTER);
+  }
+  return RefPtr<Surface>(new Surface(surface, false));
+}
+
+RefPtr<const Surface> Context::get_group_target() const
+{
+  cairo_surface_t* surface = cairo_get_group_target(m_cobject);
+  // surface can be NULL if you're not between push/pop group calls
+  if (surface == NULL)
+  {
+    // FIXME: is this really the right way to handle this?
+    throw_exception(CAIRO_STATUS_NULL_POINTER);
+  }
+  return RefPtr<const Surface>(new Surface(surface, false));
+}
+
 } //namespace Cairo
 
 // vim: ts=2 sw=2 et
