@@ -33,6 +33,11 @@ Win32Surface::~Win32Surface()
   // surface is destroyed in base class
 }
 
+HDC Win32Surface::get_dc() const
+{
+  return cairo_win32_surface_get_dc(m_cobject);
+}
+
 RefPtr<Win32Surface> Win32Surface::create(HDC hdc)
 {
   cairo_surface_t* cobject = cairo_win32_surface_create(hdc);
@@ -40,6 +45,13 @@ RefPtr<Win32Surface> Win32Surface::create(HDC hdc)
   return RefPtr<Win32Surface>(new Win32Surface(cobject, true /* has reference */));
 }
 
+RefPtr<Win32Surface> Win32Surface::create(Format format, int width, int height)
+{
+  cairo_surface_t* cobject = cairo_win32_surface_create_with_dib(format, width, height);
+  check_status_and_throw_exception(cairo_surface_status(cobject));
+  return RefPtr<Win32Surface>(new Win32Surface(cobject, true /* has reference */));
+}
+
 #endif // CAIRO_HAS_WIN32_SURFACE
 
 } //namespace Cairo
